// 2007-08-10

_fss.elementIdentified = function (id,tagName) {
	var elt = id? document.getElementById(id): null;
	if (elt) {
		if (elt._fssGenerated) // check generated ever
			elt._fssGeneratedEver = true;
		else
			elt._fssPredefined = true;
	}
	else {
		elt = document.createElement(tagName);
		elt.id = id;
		elt._fssGenerated = true;
	}
	return elt;
};
//-fxref-// _fss.elementIdentified:

_fss.setStyleSheetData = function (elt, style) {
	if (typeof style == 'object') {
		for (var p in style) elt.style[p] = style[p];
	}
	else if (typeof style == 'string') {
		elt.className = style;
	}
};
//-fxref-// _fss.setStyleSheetData:

_fss.concatenateTextNode = function (element) {
	var text = '';
	if (element.hasChildNodes()) {
		for  (var x = element.firstChild; x; x = x.nextSibling) {
			if (x.nodeName == '#text') {
				text += x.nodeValue;
			}
		}
	}
	return text;
}
//-fxref-// _fss.concatenateTextNode:

_fss.concatenateTextNodeById = function (id) {
	var element = document.getElementById(id);
	if (element) {
		return _fss.concatenateTextNode(element);
	}
	return null;
}
//-fxref-// _fss.concatenateTextNodeById: _fss.concatenateTextNode

_fss.removeTextNode = function (element) {
	var text = '';
	if (element.hasChildNodes()) {
		for  (var x = element.firstChild; x; x = x.nextSibling) {
			if (x.nodeName == '#text') {
				text += x.nodeValue;
				element.removeChild(x);
			}
		}
	}
	return text;
}
//-fxref-// _fss.removeTextNode:

// from <http://www.quirksmode.org/viewport/compatibility.html> 2007-02-17
_fss.innerSize = function () {
	var x,y;
	if (self.innerHeight) // all except Explorer
	{
		x = self.innerWidth;
		y = self.innerHeight;
	}
	else if (document.documentElement && document.documentElement.clientHeight)
		// Explorer 6 Strict Mode
	{
		x = document.documentElement.clientWidth;
		y = document.documentElement.clientHeight;
	}
	else if (document.body) // other Explorers
	{
		x = document.body.clientWidth;
		y = document.body.clientHeight;
	}
	return { x: x, y: y };
}
//-fxref-// _fss.innerSize:

_fss.scrollSize = function () {
	var x,y;
	if (self.pageYOffset) // all except Explorer
	{
		x = self.pageXOffset;
		y = self.pageYOffset;
	}
	else if (document.documentElement && document.documentElement.scrollTop)
		// Explorer 6 Strict
	{
		x = document.documentElement.scrollLeft;
		y = document.documentElement.scrollTop;
	}
	else if (document.body) // all other Explorers
	{
		x = document.body.scrollLeft;
		y = document.body.scrollTop;
	}
	return { x: x, y: y };
}
//-fxref-// _fss.scrollSize:

_fss.findContainingBlockInInlineStyle = function (elt) {
	var x = elt.parentNode;
	while (x) {
//		this._log('x.style.position:'+x.style.position);
		if (x.style && (x.style.position == 'absolute' || x.style.position == 'relative' || x.style.position == 'fixed'))
			break;
		x = x.parentNode;
	}
	return x;
}
//-fxref-// _fss.findContainingBlockInInlineStyle:

_fss.elementXY = function (element) {
	var curleft = 0, curtop = 0;
	if (element.offsetParent) {
		while (element.offsetParent) {
			curtop += element.offsetTop
			curleft += element.offsetLeft
			element = element.offsetParent;
		}
	}
	else {
		if (element.x)
			curleft += element.x;
		if (element.y)
			curtop += element.y;
	}
	return { x: curleft, y: curtop } ;
}
//-fxref-// _fss.elementXY:

// from <http://www.quirksmode.org/viewport/compatibility.html> 2007-04-01
_fss.pageSize = function () {
	var x,y;
	var test1 = document.body.scrollHeight;
	var test2 = document.body.offsetHeight
	if (test1 > test2) // all but Explorer Mac
	{
		x = document.body.scrollWidth;
		y = document.body.scrollHeight;
	}
	else // Explorer Mac;
	     //would also work in Explorer 6 Strict, Mozilla and Safari
	{
		x = document.body.offsetWidth;
		y = document.body.offsetHeight;
	}
	return { x: x, y: y };
}
//-fxref-// _fss.pageSize:

_fss.copyObject = function (from,to) {
// 1.0s: 2007-04-19 changed copyObject()
	if (to == undefined) to = new Object;
//	for (var prop in from)
//		to[prop] = from[prop];
	for (var prop in from) {
		if (typeof from[prop] == 'object')
			to[prop] = _fss.copyObject(from[prop],to[prop]);
		else
			to[prop] = from[prop];
	}
	return to;
};
//-fxref-// _fss.copyObject:

_fss.mergeObject = function (obj1,obj2) {
// 1.0t: 2007-04-19 changed mergeObject()
//	return _fss.copyObject(obj2,_fss.copyObject(obj1, new Object));
	return _fss.copyObject(obj1,_fss.copyObject(obj2));
};
//-fxref-// _fss.mergeObject: _fss.copyObject


_fss.elementObject = function (object) {
	var element = null;
	if (object) {
		if (object.elt) {
			element = object.elt;
		}
		else if (object.id || object.tagName) {
			element = _fss.elementIdentified(object.id,object.tagName);
		}
		if (element) {
			if (object.title) {
				element.title = object.title;
			}
			if (typeof object.label == 'string') {
				element.appendChild(document.createTextNode(object.label));
			}
			else if (typeof object.label == 'object') {
				element.appendChild(object.label);
			}
			_fss.setStyleSheetData(element,object.style);
		}
	}
	return element;
};
//-fxref-// _fss.elementObject: _fss.elementIdentified _fss.setStyleSheetData


_fss.styleSilhouette = {
	margin:			'1em',	// height
	backgroundColor:	'#999',
	opacity:		'0.5',
	filter:			'alpha(opacity=50)'
};
//-fxref-// _fss.styleSilhouette:

_fss.styleBody = {
	width: '10em',
	textAlign: 'center',
	padding: '1ex 1em',
	border: '1px solid #000',
	backgroundColor: '#fff',
	color: '#000'
};
//-fxref-// _fss.styleBody:

_fss.opacity = function (elt,v) {
	elt.style.opacity = v;
	elt.style.filter = 'alpha(opacity='+(v*100)+')';
};
//-fxref-// _fss.opacity:
