/*
    timer
    version 1.0 Copyright (C) 2007 Tagata Akira

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation; either version 2 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
    02110-1301 USA

*/

/****************************************************************
  Warning.  This program specifications including API may change
  because of beta version.
****************************************************************/

_fss.createUnit = function() { return new _fss.prototype._unit; };

//-fxref-// _fss.createUnit:

_fss.prototype._unit = function() {};

_fss.prototype._unit.tUniqKeyPrefix = 'fss-timer-';
_fss.prototype._unit.tUniqKeyNumber = 0;
_fss.prototype._unit.tCache = new Object;

_fss.prototype._unit.prototype.initTimer = function(tDelay,tHandler,tObject,tUniqKey) {
	if (typeof tUniqKey == 'string')
		this.tUniqKey = tUniqKey;
	else
		this.tUniqKey = _fss.prototype._unit.tUniqKeyPrefix + _fss.prototype._unit.tUniqKeyNumber++;
	this.tDelay = tDelay;
	this.tHandler = tHandler;
	this.tObject = tObject;
};

_fss.prototype._unit.tWrapper = function (tUniqKey) {
	// internal use wrapper
	if (tUniqKey in _fss.prototype._unit.tCache) {
		var tmo = _fss.prototype._unit.tCache[tUniqKey]; // timer object
		tmo.tHandler.call(tmo.tObject,tmo);
//		if (tmo.timerId)
//			delete tmo.timerId;
		_fss.prototype._unit.prototype.statusTimer.call(tmo,'wrapped',true);
	}
}

_fss.prototype._unit.prototype.setTimer = function (tDelay,tHandler,tObject,windowMethod) {
	if (tDelay != undefined)	this.tDelay = tDelay;
	if (tHandler)	this.tHandler = tHandler;
	if (tObject)	this.tObject = tObject;
	_fss.prototype._unit.tCache[this.tUniqKey] = this;
	if (this.tDelay)
		this.timerId = windowMethod('_fss.prototype._unit.tWrapper("'+this.tUniqKey+'")',this.tDelay);
	else
		_fss.prototype._unit.tWrapper(this.tUniqKey);
};


_fss.prototype._unit.prototype.clearTimer = function(windowMethod) {
	if (typeof this.timerId == 'number') {
		windowMethod(this.timerId);
		delete this.timerId;
	}
};


_fss.prototype._unit.prototype.statusTimer = function (flag,value) {
	return (typeof value == 'undefined')? this[flag]: (this[flag] = value);
};


// timeout

_fss.prototype._unit.prototype.statusTimeout = function (flag,value) {
	return this.statusTimer(flag+'TimeoutStatus',value);
}

_fss.prototype._unit.prototype.initTimeout = function(tDelay,tHandler,tObject,tUniqKey) {
	this.initTimer(tDelay,tHandler,tObject,tUniqKey);
	this.statusTimeout('init',true);
//	this.statusTimer('wrapped',false);
};

_fss.prototype._unit.prototype.setTimeout = function (tDelay,tHandler,tObject) {
	this.setTimer(tDelay,tHandler,tObject,window.setTimeout);
	this.statusTimeout('set',true);
	this.statusTimer('wrapped',false);
};

_fss.prototype._unit.prototype.clearTimeout = function() {
	this.clearTimer(window.clearTimeout);
	this.statusTimeout('clear',true);
//	this.statusTimer('wrapped',false);
}

_fss.prototype._unit.prototype.terminateTimeout = function() {
	if (!(this.statusTimeout('clear') || this.statusTimer('wrapped'))) {
		this.clearTimeout();
		_fss.prototype._unit.tWrapper(this.tUniqKey);
	}
}

// interval

_fss.prototype._unit.prototype.statusInterval = function (flag,value) {
	return this.statusTimer(flag+'IntervalStatus',value);
}

_fss.prototype._unit.prototype.initInterval = function(tDelay,tHandler,tObject,tUniqKey) {
	this.initTimer(tDelay,tHandler,tObject,tUniqKey);
	this.statusInterval('init',true);
};

_fss.prototype._unit.prototype.setInterval = function (tDelay,tHandler,tObject) {
	this.setTimer(tDelay,tHandler,tObject,window.setInterval);
	this.statusInterval('set',true);
};

_fss.prototype._unit.prototype.clearInterval = function() {
	this.clearTimer(window.clearInterval);
	this.statusInterval('clear',true);
}

////////////////////////////////////////////////////////////////

_fss.automaton = function (elt,um,period,interval) {
	this.elt = elt;
	this.um = um;
	if (period != undefined)
		this.period = period;
	if (interval != undefined)
		this.interval = interval;
	this.init();
	this.initInterval(this.interval,function() {
		this.um.call(this.elt,this.n,this.max);
		if (++this.n >= this.max) {
			this.clearInterval();
			if (this.complete) this.complete.call(this.elt);
		}
	},this);
};

_fss.automaton.prototype = _fss.createUnit();
_fss.automaton.prototype.start = function(complete) {
	if (complete)
		this.complete = complete;
	this.setInterval();
};
_fss.automaton.prototype.init = function() {
	this.n = 0;
	this.max = this.period/this.interval;
};
_fss.automaton.prototype.clear = function () {
	this.clearInterval();
	this.init();
}
_fss.automaton.prototype.period = 1000;
_fss.automaton.prototype.interval = 50;

//-fxref-// _fss.automaton: _fss.createUnit

////////////////////////////////////////////////////////////////

// 2007-04-16
_fss.prototype._unit.prototype.clearEventListener = function (event) {
	var onEvent = 'on'+event;

//	if (this.elt && onEvent in this.elt)
	if (this.elt && this.elt[onEvent])
//		this.elt[onEvent] = undefined;  // MSIE6 reports error
		this.elt[onEvent] = null;
}


_fss.prototype._unit.prototype.ignoreTextNodeEvent = true;

_fss.prototype._unit.prototype.setEventListener = function (elt,handler,event,uo) {
// 2007-04-16
	this.elt = elt;
	this.elt.$_fuo = this;
	elt['on'+event] = function (e) {
		// <http://www.quirksmode.org/js/events_properties.html> 2007-02-12
		if (!e) var e = window.event;
		var fuEvent = new Object;
		fuEvent.$uo = uo;

		// copy type
		fuEvent.type = e.type;

		// copy target
		if (e.target) fuEvent.target = e.target;
		else if (e.srcElement) fuEvent.target = e.srcElement;
// 2007-04-16
//		if (fuEvent.target.nodeType == 3) // defeat Safari bug
//			fuEvent.target = fuEvent.target.parentNode;
		if (fuEvent.target.nodeType == 3 && this.$_fuo.ignoreTextNodeEvent)
			return;

		// copy keyCode
		if (e.keyCode) fuEvent.keyCode = e.keyCode;
		else if (e.which) fuEvent.keyCode = e.which;

		// save mouse position
		fuEvent.mouseX = 0;
		fuEvent.mouseY = 0;
//		if (!e) var e = window.event;
		if (e.pageX || e.pageY) 	{
			fuEvent.mouseX = e.pageX;
			fuEvent.mouseY = e.pageY;
		}
		else if (e.clientX || e.clientY) 	{
			fuEvent.mouseX = e.clientX + document.body.scrollLeft
				+ document.documentElement.scrollLeft;
			fuEvent.mouseY = e.clientY + document.body.scrollTop
				+ document.documentElement.scrollTop;
		}


		// <http://www.quirksmode.org/js/events_mouse.html> 2007-02-12
		// copy relatedTarget
		if (fuEvent.type.toLowerCase() == 'mouseover')
			fuEvent.relatedTarget = e.relatedTarget || e.fromElement;
		else if (fuEvent.type.toLowerCase() == 'mouseout')
			fuEvent.relatedTarget = e.relatedTarget || e.toElement;
// 2007-04-16
		if (fuEvent.relatedTarget && fuEvent.relatedTarget.nodeType == 3 && this.$_fuo.ignoreTextNodeEvent)
			return;

// 2007-07-17 returns value of handler()
//		handler.call(this,fuEvent,e);
		return handler.call(this,fuEvent,e);
	}
};


_fss.prototype._unit.prototype.initDelayedEvent = function (elt,um,delay,event,umEvent) {
	if (!elt.$_fssUnits) elt.$_fssUnits = new Object;
	elt.$_fssUnits[event] = this;

	this.elt = elt;
	this.umEvent = umEvent;
	this.um = um;

	this.initTimeout(delay,
		function (fu) {
			this.um.call(this.elt,fu);
			this.executed(true);
		},this);

	this.executed = function (flag) {
		return (flag == undefined)? this.executedFlag:
			(this.executedFlag = flag);
	};
	this.executed(false); // init

	this.setEventListener(elt,function(e) {
		var fu = this.$_fssUnits[e.type];
		fu.event = e;
		if (fu.umEvent)
			if (!fu.umEvent.call(this,fu))
				return;
		fu.clearTimeout();

		if (!fu.executed())
			fu.setTimeout();
	},event);
};
